package com.sun.media.sound;

import javax.sound.midi.MidiEvent;
import java.io.ByteArrayInputStream;
import javax.sound.midi.MetaMessage;
import javax.sound.midi.SysexMessage;
import javax.sound.midi.ShortMessage;
import java.io.ByteArrayOutputStream;
import javax.sound.midi.Track;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.SequenceInputStream;
import javax.sound.midi.InvalidMidiDataException;
import java.io.FileOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import javax.sound.midi.Sequence;
import java.io.DataOutputStream;
import javax.sound.midi.spi.MidiFileWriter;

public final class StandardMidiFileWriter extends MidiFileWriter
{
    private static final int MThd_MAGIC = 1297377380;
    private static final int MTrk_MAGIC = 1297379947;
    private static final int ONE_BYTE = 1;
    private static final int TWO_BYTE = 2;
    private static final int SYSEX = 3;
    private static final int META = 4;
    private static final int ERROR = 5;
    private static final int IGNORE = 6;
    private static final int MIDI_TYPE_0 = 0;
    private static final int MIDI_TYPE_1 = 1;
    private static final int bufferSize = 16384;
    private DataOutputStream tddos;
    private static final int[] types;
    private static final long mask = 127L;
    
    public StandardMidiFileWriter() {
    }
    
    @Override
    public int[] getMidiFileTypes() {
        int[] array = new int[StandardMidiFileWriter.types.length];
        System.arraycopy(StandardMidiFileWriter.types, 0, array, 0, StandardMidiFileWriter.types.length);
        return array;
    }
    
    @Override
    public int[] getMidiFileTypes(Sequence sequence) {
        if (sequence.getTracks().length == 1) {
            return new int[] { 0, 1 };
        }
        return new int[] { 1 };
    }
    
    @Override
    public boolean isFileTypeSupported(int n) {
        int i = 0;
        while (i < StandardMidiFileWriter.types.length) {
            if (n == StandardMidiFileWriter.types[i]) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    @Override
    public int write(Sequence sequence, int fileType, OutputStream outputStream) throws IOException {
        long n = 0L;
        if (!super.isFileTypeSupported(fileType, sequence)) {
            throw new IllegalArgumentException("Could not write MIDI file");
        }
        InputStream fileStream = this.getFileStream(fileType, sequence);
        if (fileStream == null) {
            throw new IllegalArgumentException("Could not write MIDI file");
        }
        int read;
        for (byte[] array = new byte[16384]; (read = fileStream.read(array)) >= 0; n += (long)read) {
            outputStream.write(array, 0, read);
        }
        return (int)n;
    }
    
    @Override
    public int write(Sequence sequence, int n, File file) throws IOException {
        FileOutputStream fileOutputStream = new FileOutputStream(file);
        int write = this.write(sequence, n, fileOutputStream);
        fileOutputStream.close();
        return write;
    }
    
    private InputStream getFileStream(int n, Sequence sequence) throws IOException {
        Track[] tracks = sequence.getTracks();
        int n2 = 14;
        if (n == 0) {
            if (tracks.length != 1) {
                return null;
            }
        }
        else if (n == 1) {
            if (tracks.length < 1) {
                return null;
            }
        }
        else if (tracks.length == 1) {
            n = 0;
        }
        else if (tracks.length > 1) {
            n = 1;
        }
        else {
            return null;
        }
        InputStream[] array = new InputStream[tracks.length];
        int n3 = 0;
        for (int i = 0; i < tracks.length; ++i) {
            try {
                array[n3] = this.writeTrack(tracks[i], n);
                ++n3;
            }
            catch (InvalidMidiDataException ex) {}
        }
        InputStream inputStream;
        if (n3 == 1) {
            inputStream = array[0];
        }
        else if (n3 > 1) {
            inputStream = array[0];
            for (int j = 1; j < tracks.length; ++j) {
                if (array[j] != null) {
                    inputStream = new SequenceInputStream(inputStream, array[j]);
                }
            }
        }
        else {
            throw new IllegalArgumentException("invalid MIDI data in sequence");
        }
        PipedOutputStream o = new PipedOutputStream();
        DataOutputStream o2 = new DataOutputStream(o);
        PipedInputStream s1 = new PipedInputStream(o);
        o2.writeInt(1297377380);
        o2.writeInt(n2 - 8);
        if (n == 0) {
            o2.writeShort(0);
        }
        else {
            o2.writeShort(1);
        }
        o2.writeShort((int)(short)n3);
        float divisionType = sequence.getDivisionType();
        int resolution;
        if (divisionType == 0.0f) {
            resolution = sequence.getResolution();
        }
        else if (divisionType == 24.0f) {
            resolution = -6144 + (sequence.getResolution() & 0xFF);
        }
        else if (divisionType == 25.0f) {
            resolution = -6400 + (sequence.getResolution() & 0xFF);
        }
        else if (divisionType == 29.97f) {
            resolution = -7424 + (sequence.getResolution() & 0xFF);
        }
        else if (divisionType == 30.0f) {
            resolution = -7680 + (sequence.getResolution() & 0xFF);
        }
        else {
            return null;
        }
        o2.writeShort(resolution);
        SequenceInputStream o3 = new SequenceInputStream(s1, inputStream);
        o2.close();
        return o3;
    }
    
    private int getType(int n) {
        if ((n & 0xF0) == 0xF0) {
            switch (n) {
                case 240:
                case 247: {
                    return 3;
                }
                case 255: {
                    return 4;
                }
                default: {
                    return 6;
                }
            }
        }
        else {
            switch (n & 0xF0) {
                case 128:
                case 144:
                case 160:
                case 176:
                case 224: {
                    return 2;
                }
                case 192:
                case 208: {
                    return 1;
                }
                default: {
                    return 5;
                }
            }
        }
    }
    
    private int writeVarInt(long n) throws IOException {
        int n2 = 1;
        int i;
        for (i = 63; i > 0 && (n & 127L << i) == 0x0L; i -= 7) {}
        while (i > 0) {
            this.tddos.writeByte((int)((n & 127L << i) >> i | 0x80L));
            i -= 7;
            ++n2;
        }
        this.tddos.writeByte((int)(n & 0x7FL));
        return n2;
    }
    
    private InputStream writeTrack(Track track, int n) throws IOException, InvalidMidiDataException {
        int v = 0;
        int size = track.size();
        PipedOutputStream pipedOutputStream = new PipedOutputStream();
        DataOutputStream dataOutputStream = new DataOutputStream(pipedOutputStream);
        PipedInputStream s1 = new PipedInputStream(pipedOutputStream);
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        this.tddos = new DataOutputStream(out);
        long tick = 0L;
        int n2 = -1;
        for (int i = 0; i < size; ++i) {
            MidiEvent value = track.get(i);
            value.getTick();
            long n3 = value.getTick() - tick;
            tick = value.getTick();
            int status = value.getMessage().getStatus();
            switch (this.getType(status)) {
                case 1: {
                    int data1 = ((ShortMessage)value.getMessage()).getData1();
                    v += this.writeVarInt(n3);
                    if (status != n2) {
                        n2 = status;
                        this.tddos.writeByte(status);
                        ++v;
                    }
                    this.tddos.writeByte(data1);
                    ++v;
                    break;
                }
                case 2: {
                    ShortMessage shortMessage = (ShortMessage)value.getMessage();
                    int data2 = shortMessage.getData1();
                    int data3 = shortMessage.getData2();
                    v += this.writeVarInt(n3);
                    if (status != n2) {
                        n2 = status;
                        this.tddos.writeByte(status);
                        ++v;
                    }
                    this.tddos.writeByte(data2);
                    ++v;
                    this.tddos.writeByte(data3);
                    ++v;
                    break;
                }
                case 3: {
                    SysexMessage sysexMessage = (SysexMessage)value.getMessage();
                    sysexMessage.getLength();
                    byte[] message = sysexMessage.getMessage();
                    int n4 = v + this.writeVarInt(n3);
                    n2 = status;
                    this.tddos.writeByte((int)message[0]);
                    int n5 = ++n4 + this.writeVarInt((long)(message.length - 1));
                    this.tddos.write(message, 1, message.length - 1);
                    v = n5 + (message.length - 1);
                    break;
                }
                case 4: {
                    MetaMessage metaMessage = (MetaMessage)value.getMessage();
                    metaMessage.getLength();
                    byte[] message2 = metaMessage.getMessage();
                    int n6 = v + this.writeVarInt(n3);
                    n2 = status;
                    this.tddos.write(message2, 0, message2.length);
                    v = n6 + message2.length;
                    break;
                }
                case 6: {
                    break;
                }
                case 5: {
                    break;
                }
                default: {
                    throw new InvalidMidiDataException("internal file writer error");
                }
            }
        }
        dataOutputStream.writeInt(1297379947);
        dataOutputStream.writeInt(v);
        v += 8;
        SequenceInputStream o = new SequenceInputStream(s1, new ByteArrayInputStream(out.toByteArray()));
        dataOutputStream.close();
        this.tddos.close();
        return o;
    }
    
    static {
        types = new int[] { 0, 1 };
    }
}
